using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;

namespace SeperateModelTest
{
    public partial class Form1 : Form
    {
        private ExternalData _extData;
        private GridModel _model;
        private GridFormulaEngine engine;
        CustomFormulaCellRenderer renderer;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            _model = new GridModel();
            _model.CellModels.Add("FormulaCell", new CustomFormulaCellModel(_model)); //uses new custom celltype
            _model.BaseStylesMap["Standard"].StyleInfo.CellType = "FormulaCell";
             
            _extData = new ExternalData(100, 20);
            _model.QueryCellInfo += new Syncfusion.Windows.Forms.Grid.GridQueryCellInfoEventHandler(GridQueryCellInfo);
            _model.QueryRowCount += new Syncfusion.Windows.Forms.Grid.GridRowColCountEventHandler(GridQueryRowCount);
            _model.QueryColCount += new Syncfusion.Windows.Forms.Grid.GridRowColCountEventHandler(GridQueryColCount);
            _model.SaveCellInfo += new GridSaveCellInfoEventHandler(GridSaveCellInfo);
           
            
            _model.Rows.DefaultSize = 19;
            _model.Cols.DefaultSize = 65;

            gridControl1.Model = _model;
            gridControl1.CurrentCellValidating += new CancelEventHandler(gridControl1_CurrentCellValidating);
            
            renderer = (CustomFormulaCellRenderer)gridControl1.CellRenderers["FormulaCell"];
            engine = ((GridFormulaCellModel)gridControl1.Model.CellModels["FormulaCell"]).Engine;
            renderer.IgnoreSaveCellInfo = false;
          
        }

        void gridControl1_CurrentCellValidating(object sender, CancelEventArgs e)
        {
            GridControl grid = sender as GridControl;
            GridCurrentCell cc = grid.CurrentCell;
            if (cc.Renderer.ControlText.StartsWith("="))
            {
                _extData[cc.RowIndex - 1, cc.ColIndex - 1] = new WithFormula();
            }
        }

        public void GridQueryCellInfo(object sender, GridQueryCellInfoEventArgs e)
        {
            if (e.RowIndex > 0 && e.ColIndex > 0)
            {
                e.Style.CellValue = _extData[e.RowIndex - 1, e.ColIndex - 1].value;
                e.Style.FormulaTag = _extData[e.RowIndex - 1, e.ColIndex - 1].tag;
                e.Handled = true;
            }
        }

        public void GridQueryRowCount(object sender, GridRowColCountEventArgs e)
        {
            e.Count = this._extData.RowCount;
            e.Handled = true;
        }

        public void GridQueryColCount(object sender, GridRowColCountEventArgs e)
        {
            e.Count = this._extData.ColCount;
            e.Handled = true;
        }

        public void GridSaveCellInfo(object sender, GridSaveCellInfoEventArgs e)
        {
            if (!renderer.IgnoreSaveCellInfo)
            {

                Console.WriteLine("GridSaveCellInfo hit {0},{1}.", e.RowIndex, e.ColIndex);
                try
                {
                    if (e.ColIndex > 0 && e.RowIndex > 0)
                    {
                        WithFormula withFormula = new WithFormula();
                        withFormula.value = e.Style.CellValue.ToString();
                        if (e.Style.FormulaTag != null)
                        {
                            withFormula.tag = e.Style.FormulaTag;
                        }
                        else if (withFormula.value.StartsWith("="))
                        {
                            string parsedFormula = engine.Parse(withFormula.value.Substring(1));
                            string computedValue = engine.ComputedValue(parsedFormula);
                            withFormula.tag = new GridFormulaTag(parsedFormula, computedValue, e.RowIndex, e.ColIndex);
                        }

                        _extData[e.RowIndex - 1, e.ColIndex - 1] = withFormula;
                    }
                }
                catch { }
            }
            e.Handled = true;
        }
    }

  
}